--- ========================================================= ---
---  Setup Locals
--- ========================================================= ---
local DaHud = LibStub("AceAddon-3.0"):GetAddon("DaHud")

local localeTables = {}
function DaHud:L(name)
	if not localeTables[name] then
		localeTables[name] = setmetatable({}, {__index = function(self, key)
			self[key] = key
			return key
		end})
	end
	return localeTables[name]
end

local L = DaHud:L("DaHud")

--- ========================================================= ---
---  API
--- ========================================================= ---
DaHud.Style.Default = L["Default"]
DaHud.Style.Mini = L["Mini"]
DaHud.Style.Informational = L["Informational"]

--- ========================================================= ---
---  Styles
--- ========================================================= ---
DaHud.Styles.Bars.Default = {
	bar1 = {
		name = "PlayerHealth",
		active = true,
		posx = -155,
		posy = 0,
		unit = "player",
		type = "health",
		border = "dahud_left",
	},
	bar2 = {
		name = "PlayerMana",
		active = true,
		posx = -135,
		posy = 0,
		unit = "player",
		type = "power",
		border = "dahud_right",
	},
	bar3 = {
		name = "TargetHealth",
		active = true,
		posx = 155,
		posy = 0,
		unit = "target",
		type = "health",
		border = "dahud_right",
	},
	bar4 = {
		name = "TargetMana",
		active = true,
		posx = 135,
		posy = 0,
		unit = "target",
		type = "power",
		border = "dahud_left",
	},
	bar5 = {
		name = "PlayerHealthCasting",
		active = true,
		framelevel = 2,
		anchor = "DaHud_bar1",
		posx = 0,
		posy = 0,
		unit = "player",
		type = "casting",
		background = "custom",
		border = "custom",
		bar = "dahud_cbleft",
	},
	bar6 = {
		name = "PlayerManaCasting",
		active = true,
		framelevel = 2,
		anchor = "DaHud_bar2",
		posx = 0,
		posy = 0,
		unit = "player",
		type = "casting",
		background = "custom",
		border = "custom",
		bar = "dahud_cbright",
	},
	bar7 = {
		name = "TargetHealthCasting",
		active = true,
		framelevel = 2,
		anchor = "DaHud_bar3",
		posx = 0,
		posy = 0,
		unit = "target",
		type = "casting",
		background = "custom",
		border = "custom",
		bar = "dahud_cbright",
	},
	bar8 = {
		name = "TargetManaCasting",
		active = true,
		framelevel = 2,
		anchor = "DaHud_bar4",
		posx = 0,
		posy = 0,
		unit = "target",
		type = "casting",
		background = "custom",
		border = "custom",
		bar = "dahud_cbleft",
	},
}

DaHud.Styles.Text.Default = {
	text1 = {
		name = "PlayerHealth",
		active = true,
		posx = -210,
		posy = -135,
		unit = "player",
		fontalign = "RIGHT",
		fonttype = "Health",
		fontstyle = "Absolute and Percent",
	},
	text2 = {
		name = "PlayerMana",
		active = true,
		posx = -210,
		posy = -155,
		unit = "player",
		fontalign = "RIGHT",
		fonttype = "Power",
		fontstyle = "Absolute and Percent",
	},
	text3 = {
		name = "TargetHealth",
		active = true,
		posx = 210,
		posy = -135,
		unit = "target",
		fontalign = "LEFT",
		fonttype = "Health",
		fontstyle = "Absolute and Percent",
	},
	text4 = {
		name = "TargetMana",
		active = true,
		posx = 210,
		posy = -155,
		unit = "target",
		fontalign = "LEFT",
		fonttype = "Power",
		fontstyle = "Absolute and Percent",
	},
	text5 = {
		name = "PlayerCastingName",
		active = true,
		posx = -235,
		posy = 135,
		unit = "player",
		fontalign = "RIGHT",
		fonttype = "Cast",
		fontstyle = "Standard Name",
	},
	text6 = {
		name = "PlayerCastingTime",
		active = true,
		posx = -55,
		posy = 135,
		unit = "player",
		fontalign = "LEFT",
		fonttype = "Cast",
		fontstyle = "Standard Time",
	},
	text7 = {
		name = "TargetCastingName",
		active = true,
		posx = 235,
		posy = 135,
		unit = "target",
		fontalign = "LEFT",
		fonttype = "Cast",
		fontstyle = "Standard Name",
	},
	text8 = {
		name = "TargetCastingTime",
		active = true,
		posx = 55,
		posy = 135,
		unit = "target",
		fontalign = "RIGHT",
		fonttype = "Cast",
		fontstyle = "Standard Time",
	},
}

DaHud.Styles.Icons.Default = {
	icon1 = {
		name = "PlayerResting",
		active = true,
		posx = -107,
		posy = 105,
		unit = "player",
		type = "resting",
	},
	icon2 = {
		name = "PlayerCombat",
		active = true,
		posx = -107,
		posy = 105,
		unit = "player",
		type = "combat",
	},
	icon3 = {
		name = "PlayerPvPFlag",
		active = true,
		posx = -107,
		posy = 80,
		unit = "player",
		type = "pvpflag",
	},
	icon4 = {
		name = "PlayerLeader",
		active = true,
		posx = -107,
		posy = 60,
		unit = "player",
		type = "leader",
	},
	icon5 = {
		name = "PlayerMasterLooter",
		active = true,
		posx = -107,
		posy = 40,
		unit = "player",
		type = "masterlooter",
	},
	icon6 = {
		name = "PlayerRaidTarget",
		active = true,
		posx = -145,
		posy = 115,
		width = 20,
		height = 20,
		unit = "player",
		type = "raidtarget",
	},
	icon7 = {
		name = "TargetCombat",
		active = true,
		posx = 107,
		posy = 105,
		unit = "target",
		type = "combat",
	},
	icon8 = {
		name = "TargetPvP",
		active = true,
		posx = 107,
		posy = 80,
		unit = "target",
		type = "pvpflag",
	},
	icon9 = {
		name = "TargetRaidTarget",
		active = true,
		posx = 145,
		posy = 115,
		width = 20,
		height = 20,
		unit = "target",
		type = "raidtarget",
	},
	icon10 = {
		name = "TargetComboPoints",
		active = true,
		point = "TOP",
		posx = 180,
		posy = 110,
		width = 18,
		height = 15,
		unit = "target",
		type = "combopoints",
		style = "dahud_combopoints",
	},
}

DaHud.Styles.Buffs.Default = {
	buff1 = {
		name = "TargetBuffs",
		active = true,
		posx = -115,
		posy = -190,
		width = 20,
		height = 20,
		unit = "target",
		type = "buffs",
		direction = "LTD",
		colorDebuff = true,
		large = true,
		tooltip = "always",
	},
	buff2 = {
		name = "TargetDebuffs",
		active = true,
		posx = 115,
		posy = -190,
		width = 20,
		height = 20,
		unit = "target",
		type = "debuffs",
		direction = "RTD",
		colorDebuff = true,
		large = true,
		tooltip = "always",
	},
}

DaHud.Styles.Bars.Mini = {
	bar1 = {
		name = "PlayerHealth",
		active = true,
		posx = -155,
		posy = 0,
		unit = "player",
		type = "health",
		border = "dahud_left",
	},
	bar2 = {
		name = "PlayerMana",
		active = true,
		posx = -135,
		posy = 0,
		unit = "player",
		type = "power",
		border = "dahud_right",
	},
	bar3 = {
		name = "TargetHealth",
		active = true,
		posx = 155,
		posy = 0,
		unit = "target",
		type = "health",
		border = "dahud_right",
	},
	bar4 = {
		name = "TargetMana",
		active = true,
		posx = 135,
		posy = 0,
		unit = "target",
		type = "power",
		border = "dahud_left",
	},
	bar5 = {
		name = "PlayerHealthCasting",
		active = true,
		framelevel = 2,
		anchor = "DaHud_bar1",
		posx = 0,
		posy = 0,
		unit = "player",
		type = "casting",
		background = "custom",
		border = "custom",
		bar = "dahud_cbleft",
	},
	bar6 = {
		name = "PlayerManaCasting",
		active = true,
		framelevel = 2,
		anchor = "DaHud_bar2",
		posx = 0,
		posy = 0,
		unit = "player",
		type = "casting",
		background = "custom",
		border = "custom",
		bar = "dahud_cbright",
	},
	bar7 = {
		name = "TargetHealthCasting",
		active = true,
		framelevel = 2,
		anchor = "DaHud_bar3",
		posx = 0,
		posy = 0,
		unit = "target",
		type = "casting",
		background = "custom",
		border = "custom",
		bar = "dahud_cbright",
	},
	bar8 = {
		name = "TargetManaCasting",
		active = true,
		framelevel = 2,
		anchor = "DaHud_bar4",
		posx = 0,
		posy = 0,
		unit = "target",
		type = "casting",
		background = "custom",
		border = "custom",
		bar = "dahud_cbleft",
	},
	bar9 = {
		name = "PlayerThreat",
		active = true,
		posx = -190,
		posy = 0,
		unit = "player",
		type = "threat",
		border = "dahud_double",
	},
	bar10 = {
		name = "DruidMana",
		active = true,
		posx = -113,
		posy = -60,
		width = 20,
		height = 125,
		unit = "player",
		type = "druidmana",
		border = "dahud_center",
	},
}

DaHud.Styles.Text.Mini = {
	text1 = {
		name = "PlayerHealth",
		active = true,
		posx = -210,
		posy = -135,
		unit = "player",
		fontalign = "RIGHT",
		fonttype = "Health",
		fontstyle = "Absolute and Percent",
	},
	text2 = {
		name = "PlayerMana",
		active = true,
		posx = -210,
		posy = -155,
		unit = "player",
		fontalign = "RIGHT",
		fonttype = "Power",
		fontstyle = "Absolute and Percent",
	},
	text3 = {
		name = "TargetHealth",
		active = true,
		posx = 210,
		posy = -135,
		unit = "target",
		fontalign = "LEFT",
		fonttype = "Health",
		fontstyle = "Absolute and Percent",
	},
	text4 = {
		name = "TargetMana",
		active = true,
		posx = 210,
		posy = -155,
		unit = "target",
		fontalign = "LEFT",
		fonttype = "Power",
		fontstyle = "Absolute and Percent",
	},
	text5 = {
		name = "PlayerCastingName",
		active = true,
		posx = -235,
		posy = 135,
		unit = "player",
		fontalign = "RIGHT",
		fonttype = "Cast",
		fontstyle = "Standard Name",
	},
	text6 = {
		name = "PlayerCastingTime",
		active = true,
		posx = -55,
		posy = 135,
		unit = "player",
		fontalign = "LEFT",
		fonttype = "Cast",
		fontstyle = "Standard Time",
	},
	text7 = {
		name = "TargetCastingName",
		active = true,
		posx = 235,
		posy = 135,
		unit = "target",
		fontalign = "LEFT",
		fonttype = "Cast",
		fontstyle = "Standard Name",
	},
	text8 = {
		name = "TargetCastingTime",
		active = true,
		posx = 55,
		posy = 135,
		unit = "target",
		fontalign = "RIGHT",
		fonttype = "Cast",
		fontstyle = "Standard Time",
	},
}

DaHud.Styles.Icons.Mini = {
	icon1 = {
		name = "PlayerResting",
		active = true,
		posx = -107,
		posy = 105,
		unit = "player",
		type = "resting",
	},
	icon2 = {
		name = "PlayerCombat",
		active = true,
		posx = -107,
		posy = 105,
		unit = "player",
		type = "combat",
	},
	icon3 = {
		name = "PlayerPvPFlag",
		active = true,
		posx = -107,
		posy = 80,
		unit = "player",
		type = "pvpflag",
	},
	icon4 = {
		name = "PlayerLeader",
		active = true,
		posx = -107,
		posy = 60,
		unit = "player",
		type = "leader",
	},
	icon5 = {
		name = "PlayerMasterLooter",
		active = true,
		posx = -107,
		posy = 40,
		unit = "player",
		type = "masterlooter",
	},
	icon6 = {
		name = "PlayerRaidTarget",
		active = true,
		posx = -145,
		posy = 115,
		width = 20,
		height = 20,
		unit = "player",
		type = "raidtarget",
	},
	icon7 = {
		name = "TargetCombat",
		active = true,
		posx = 107,
		posy = 105,
		unit = "target",
		type = "combat",
	},
	icon8 = {
		name = "TargetPvP",
		active = true,
		posx = 107,
		posy = 80,
		unit = "target",
		type = "pvpflag",
	},
	icon9 = {
		name = "TargetRaidTarget",
		active = true,
		posx = 145,
		posy = 115,
		width = 20,
		height = 20,
		unit = "target",
		type = "raidtarget",
	},
	icon10 = {
		name = "TargetComboPoints",
		active = true,
		point = "TOP",
		posx = 180,
		posy = 110,
		width = 18,
		height = 15,
		unit = "target",
		type = "combopoints",
		style = "dahud_combopoints",
	},
}

DaHud.Styles.Buffs.Mini = {
	buff1 = {
		name = "TargetBuffs",
		active = true,
		posx = -115,
		posy = -190,
		width = 20,
		height = 20,
		unit = "target",
		type = "buffs",
		direction = "LTD",
		colorDebuff = true,
		large = true,
		tooltip = "always",
	},
	buff2 = {
		name = "TargetDebuffs",
		active = true,
		posx = 115,
		posy = -190,
		width = 20,
		height = 20,
		unit = "target",
		type = "debuffs",
		direction = "RTD",
		colorDebuff = true,
		large = true,
		tooltip = "always",
	},
}

DaHud.Styles.Bars.Informational = {
	bar1 = {
		name = "PlayerHealth",
		active = true,
		posx = -155,
		posy = 0,
		unit = "player",
		type = "health",
		border = "dahud_left",
	},
	bar2 = {
		name = "PlayerMana",
		active = true,
		posx = -135,
		posy = 0,
		unit = "player",
		type = "power",
		border = "dahud_right",
	},
	bar3 = {
		name = "TargetHealth",
		active = true,
		posx = 155,
		posy = 0,
		unit = "target",
		type = "health",
		border = "dahud_right",
	},
	bar4 = {
		name = "TargetMana",
		active = true,
		posx = 135,
		posy = 0,
		unit = "target",
		type = "power",
		border = "dahud_left",
	},
	bar5 = {
		name = "PlayerHealthCasting",
		active = true,
		framelevel = 2,
		anchor = "DaHud_bar1",
		posx = 0,
		posy = 0,
		unit = "player",
		type = "casting",
		background = "custom",
		border = "custom",
		bar = "dahud_cbleft",
	},
	bar6 = {
		name = "PlayerManaCasting",
		active = true,
		framelevel = 2,
		anchor = "DaHud_bar2",
		posx = 0,
		posy = 0,
		unit = "player",
		type = "casting",
		background = "custom",
		border = "custom",
		bar = "dahud_cbright",
	},
	bar7 = {
		name = "TargetHealthCasting",
		active = true,
		framelevel = 2,
		anchor = "DaHud_bar3",
		posx = 0,
		posy = 0,
		unit = "target",
		type = "casting",
		background = "custom",
		border = "custom",
		bar = "dahud_cbright",
	},
	bar8 = {
		name = "TargetManaCasting",
		active = true,
		framelevel = 2,
		anchor = "DaHud_bar4",
		posx = 0,
		posy = 0,
		unit = "target",
		type = "casting",
		background = "custom",
		border = "custom",
		bar = "dahud_cbleft",
	},
	bar9 = {
		name = "PlayerThreat",
		active = true,
		posx = -190,
		posy = 0,
		unit = "player",
		type = "threat",
		border = "dahud_double",
	},
	bar10 = {
		name = "DruidMana",
		active = true,
		posx = -113,
		posy = -60,
		width = 20,
		height = 125,
		unit = "player",
		type = "druidmana",
		border = "dahud_center",
	},	
	bar11 = {
		name = "PetHealth",
		active = true,
		posx = -103,
		posy = -60,
		width = 20,
		height = 125,
		unit = "pet",
		type = "health",
		border = "dahud_right",
	},
	bar12 = {
		name = "PetMana",
		active = true,
		posx = -113,
		posy = -60,
		width = 20,
		height = 125,
		unit = "pet",
		type = "power",
		border = "dahud_center",
	},
	bar13 = {
		name = "FocusHealth",
		active = true,
		posx = 103,
		posy = -60,
		width = 20,
		height = 125,
		unit = "focus",
		type = "health",
		border = "dahud_left",
	},
	bar14 = {
		name = "FocusMana",
		active = true,
		posx = 113,
		posy = -60,
		width = 20,
		height = 125,
		unit = "focus",
		type = "power",
		border = "dahud_center",
	},
	bar15 = {
		name = "ToTHealth",
		active = true,
		posx = 0,
		posy = -110,
		width = 120,
		height = 20,
		unit = "targettarget",
		type = "health",
		style = "horizontal",
		border = "",
		bar = "Blizzard",
	},
}

DaHud.Styles.Text.Informational = {
	text1 = {
		name = "PlayerHealth",
		active = true,
		posx = -210,
		posy = -135,
		unit = "player",
		fontalign = "RIGHT",
		fonttype = "Health",
		fontstyle = "Absolute and Percent",
	},
	text2 = {
		name = "PlayerMana",
		active = true,
		posx = -210,
		posy = -155,
		unit = "player",
		fontalign = "RIGHT",
		fonttype = "Power",
		fontstyle = "Absolute and Percent",
	},
	text3 = {
		name = "TargetHealth",
		active = true,
		posx = 210,
		posy = -135,
		unit = "target",
		fontalign = "LEFT",
		fonttype = "Health",
		fontstyle = "Absolute and Percent",
		},
	text4 = {
		name = "TargetMana",
		active = true,
		posx = 210,
		posy = -155,
		unit = "target",
		fontalign = "LEFT",
		fonttype = "Power",
		fontstyle = "Absolute and Percent",
	},
	text5 = {
		name = "PlayerCastingName",
		active = true,
		posx = -235,
		posy = 135,
		unit = "player",
		fontalign = "RIGHT",
		fonttype = "Cast",
		fontstyle = "Standard Name",
	},
	text6 = {
		name = "PlayerCastingTime",
		active = true,
		posx = -55,
		posy = 135,
		unit = "player",
		fontalign = "LEFT",
		fonttype = "Cast",
		fontstyle = "Standard Time",
	},
	text7 = {
		name = "TargetCastingName",
		active = true,
		posx = 235,
		posy = 135,
		unit = "target",
		fontalign = "LEFT",
		fonttype = "Cast",
		fontstyle = "Standard Name",
	},
	text8 = {
		name = "TargetCastingTime",
		active = true,
		posx = 55,
		posy = 135,
		unit = "target",
		fontalign = "RIGHT",
		fonttype = "Cast",
		fontstyle = "Standard Time",
	},
	text9 = {
		name = "TargetName",
		active = true,
		posx = 0,
		posy = -135,
		unit = "target",
		fontalign = "CENTER",
		fonttype = "Name",
		fontstyle = "Long w/ Druid form",
	},
	text10 = {
		name = "TargetGuild",
		active = true,
		posx = 0,
		posy = -155,
		unit = "target",
		fontalign = "CENTER",
		fonttype = "Guild",
		fontstyle = "Standard",
	},
	text11 = {
		name = "TargetRange",
		active = true,
		posx = 0,
		posy = -175,
		unit = "target",
		fontalign = "CENTER",
		fonttype = "Range",
		fontstyle = "Standard",
	},
	text12 = {
		name = "TotName",
		active = true,
		framestrata = "MEDIUM",
		posx = 0,
		posy = -110,
		width = 118,
		height = 18,
		unit = "targettarget",
		fontalign = "CENTER",
		fonttype = "Name",
		fontstyle = "Class Colored",
	},
}

DaHud.Styles.Icons.Informational = {
	icon1 = {
		name = "PlayerResting",
		active = true,
		posx = -107,
		posy = 105,
		unit = "player",
		type = "resting",
	},
	icon2 = {
		name = "PlayerCombat",
		active = true,
		posx = -107,
		posy = 105,
		unit = "player",
		type = "combat",
	},
	icon3 = {
		name = "PlayerPvPFlag",
		active = true,
		posx = -107,
		posy = 80,
		unit = "player",
		type = "pvpflag",
	},
	icon4 = {
		name = "PlayerLeader",
		active = true,
		posx = -107,
		posy = 60,
		unit = "player",
		type = "leader",
	},
	icon5 = {
		name = "PlayerMasterLooter",
		active = true,
		posx = -107,
		posy = 40,
		unit = "player",
		type = "masterlooter",
	},
	icon6 = {
		name = "PlayerRaidTarget",
		active = true,
		posx = -145,
		posy = 115,
		width = 20,
		height = 20,
		unit = "player",
		type = "raidtarget",
	},
	icon7 = {
		name = "TargetCombat",
		active = true,
		posx = 107,
		posy = 105,
		unit = "target",
		type = "combat",
	},
	icon8 = {
		name = "TargetPvP",
		active = true,
		posx = 107,
		posy = 80,
		unit = "target",
		type = "pvpflag",
	},
	icon9 = {
		name = "TargetRaidTarget",
		active = true,
		posx = 145,
		posy = 115,
		width = 20,
		height = 20,
		unit = "target",
		type = "raidtarget",
	},
	icon10 = {
		name = "TargetComboPoints",
		active = true,
		point = "TOP",
		posx = 180,
		posy = 110,
		width = 18,
		height = 15,
		unit = "target",
		type = "combopoints",
		style = "dahud_combopoints",
	},
	icon11 = {
		name = "PetRaidTarget",
		active = true,
		posx = 0,
		posx = -107,
		width = 20,
		height = 20,
		unit = "pet",
		type = "raidtarget",
	},
	icon12 = {
		name = "FocusRaidTarget",
		active = true,
		posx = 107,
		posy = 0,
		width = 20,
		height = 20,
		unit = "focus",
		type = "raidtarget",
	},
	icon13 = {
		name = "ToTRaidTarget",
		active = true,
		posx = 0,
		posy = -100,
		width = 20,
		height = 20,
		unit = "targettarget",
		type = "raidtarget",
	},
}

DaHud.Styles.Buffs.Informational = {
	buff1 = {
		name = "TargetBuffs",
		active = true,
		posx = -115,
		posy = -190,
		width = 20,
		height = 20,
		unit = "target",
		type = "buffs",
		direction = "LTD",
		colorDebuff = true,
		large = true,
		tooltip = "always",
	},
	buff2 = {
		name = "TargetDebuffs",
		active = true,
		posx = 115,
		posy = -190,
		width = 20,
		height = 20,
		unit = "target",
		type = "debuffs",
		direction = "RTD",
		colorDebuff = true,
		large = true,
		tooltip = "always",
	},
}